# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from pyseaweed.seaweed import SeaweedFS  # noqa: F401
from pyseaweed.version import __version__

WeedFS = SeaweedFS  # for backward compatibilty

VERSION = __version__
